#!/bin/sh
#
# Update a FreeBSD system to a requested cvs version.
# 
# TODO: File permissions on files in ~/.sysupdate should not be world readable
#
# fastest_cvs command line: fastest_cvsup -c au,nz,jp -Q

# Script vars
VERSION="0.4.1 (06.Jan.07)"
SYSUPDATE=$(basename $0)
WWW="http://lvl.sourceforge.net/"
# debug can also be called with -D
#DEBUG="YES"
# DEV site 
VERSIONCHKURL="http://lvl.sourceforge.net/dl/scripts/sysupdate.ver.txt"
#scriptMD5="http://lvl.sourceforge.net/dl/scripts/sysupdate.md5"
# cvsup is located ... 
CVSUPAPP=`which cvsup`
# Set the default profile
PROFILECONF="$HOME/.sysupdate/sysupdate.conf"
SCRIPTNAME="sysupdate.sh"

# -------
# Is fastest_cvs installed?
FASTESTCVSUP=`which fastest_cvsup`
# -------

funcUsage () {
	echo "
Usage: $SYSUPDATE [-C | -h | -V | -v] [-Dmq] [-jN] [-p /path/to/conf]

 -C       Clean. Remove settings and configured options, then exit. If 
          a profile is passed using -p, that profile will be removed.
 -D       Debug mode. No file fetching, compiles or system changes are 
          performed.
 -h       Help. Display this wonderful help and exit.
 -jN      Use 'make -jN buildworld'. 'N' is a required number.
 -m       NOT SUPPORTED. Multi-user mode. Perform 'make buildworld' in 
          multi-user mode.
 -p file  Custom profile. Use the settings in 'file' instead of the 
          default config. If 'file' is not found, it will be created.
 -q       Quiet mode, ask no questions (unless there are no settings), 
          use previous settings and runs mergemaster (if required) with 
          the -ai -t flags.
 -V       Check if there is a more recent version.
 -v       Display version info and exit.

See 'man $SYSUPDATE' for more information.
version: $VERSION website: $WWW
"
	exit 0
}

funcVersion () {
	# display version info and exit
	echo "
$SYSUPDATE from $WWW
version: $VERSION
"
	exit 0
}

funcVersionCheck () {
	# check to see what the latest version is
	echo "-----------------------------------------------------------------------
Getting latest version information..."
	latestVersion=`fetch -q -T 30 -o - $VERSIONCHKURL`

	if [ "$latestVersion" = "" -o "$latestVersion" = "fetch: -: Not Found" ]
	then # unable to confirm latest version
		echo "PROBLEM: Unable to obtain latest version information: `date`"
	else
		echo "
Installed version: $VERSION
Latest version: $latestVersion
"
	fi
	exit 0
}

funcUidCheck () {
	# check user id (should be uid 0 for root) and give a warning if not
	if [ `id -u` != 0 ]
	then
	        echo "--- WARNING ---
$SYSUPDATE should be run by root. Unless your permissions have been 
modified, you will not have enough access rights to update the system.

CTRL+C to cancel now."
			sleep 1
	fi
}

funcCleanUp () {

	# Remove config options and temp files 
	rm -f "$PROFILECONF"
	rm -f "$HOME/.sysupdate/srcCustom"
	rm -f "$HOME/.sysupdate/portsCustom"
	rm -f "$HOME/.sysupdate/status"
	rm -f "$HOME/.sysupdate/lastupdate"
	rm -f "$HOME/.sysupdate/cvsupPorts"
	rm -f "$HOME/.sysupdate/cvsupSource"
		
	echo "
--- INFORMATION ---
Your settings have been reset to the default values.
-------------------"
	exit 0
}

funcSystemModeChk () {
	# work out single user mode and if it is safe to run in quiet mode
	SYSTEMMODE=`who am i`
	case "$SYSTEMMODE" in
		*" "ttyp*)
			SYSTEMMODE="MULTIUSER"
			# NOTE: Running from within X-Window will return this option (which is correct)
			# echo "INFO: System is in Multi User Mode." 
			# echo "INFO: Script is run from a remote or X-Window terminal"
			;;
		*" "ttyv*)
			SYSTEMMODE="MULTIUSER"
			# echo "INFO: System is in Multi User Mode." 
			# echo "INFO: Script is run from a local console" 
			;;
		root*" "console*)
			SYSTEMMODE="SINGLEUSER"
			# echo "INFO: System is in Single User Mode." 
			;;
		*)
			SYSTEMMODE="MULTIUSER"
			# assume a multi user system
			# echo "INFO: Can not tell system mode. (Contact author please)" 
			;;
	esac
} # funcSystemModeChk

funcJValueChk () {
	if [ "$JVALUE" != "" ]
	then
		jCHK=$(echo $JVALUE | grep -c -e [^[:digit:]])
		if [ "$jCHK" != "0" -o "$JVALUE" = "0" ]
		then
			if [ "$QUIETMODE" = "YES" ]
			then
				# we just clear the error and continue
				echo "INFO: Removing -j option because it is not valid."
				JVALUE=""
				JOPTION="NO"
			else
				echo "--- PROBLEM ---
You have entered an invalid number ($JVALUE) for the '-j' option.
"
				if [ "$QUIETMODE" != "YES" ]
				then
					echo -n "Enter a valid number or 'C' to cancel the use of '-j' : " 
					read JVALUE
				fi
				if [ "$JVALUE" = "c" -o "$JVALUE" = "C" -o "$JVALUE" = "" ]
				then
					echo "INFO: Removing -j option."
					JVALUE=""
					JOPTION="NO"
				else
					jCHK=$(echo $JVALUE | grep -c -e [^[:digit:]])
					if [ "$jCHK" != "0" -o "$JVALUE" = "0" -o "$JVALUE" = "" ]
					then    # they still did not get it right or canceled 
						funcJValueChk
					fi # re-check
				fi # c or C
			fi # QUIETMODE
		fi # jCHK
	fi #JVALUE ?
} # funcJValueChk

funcLastUpdate () {
	# pull in the last update info and present some stats
	# (the sysupdate.conf has already been pulled in)
	. "$HOME/.sysupdate/lastupdate"
	#. "$PROFILECONF"
	. "$LASTPROFILECONF"

	echo "-----------------------------------------------------------------------
 The last update (using $SCRIPTNAME) was as follows:
 Profile settings : $LASTPROFILECONF
 Date of update: $LASTUPDATE"
 	
	if [ "$UPPORTS" = "none" ]
	then
		echo " Updated the ports: No"
	elif [ "$UPPORTS" = "ports-all" ]
	then	# full update
		echo " Updated the ports: Yes"
	elif [ "$UPPORTS" = "custom" ]
	then	# custom update
		echo " Updated the ports: Yes (custom selection)"
	fi # UPPORTS

	if [ "$UPSOURCE" = "src-all" -o "$UPSOURCE" = "custom" ]
	then
		if [ "$UPSOURCE" = "custom" ]
		then
			echo " Updated system source files: Yes (custom selection)"
		else
			echo " Updated system source files: Yes"
		fi
		case "$UPRELTAG" in 
			NONE)
				;;
			RELENG_6)
				echo " cvs version: FreeBSD 6-STABLE (RELENG_6)" ;; 
			RELENG_6_2)
				echo " cvs version: FreeBSD 6.2 (RELENG_6_2)" ;; 
			RELENG_6_1)
				echo " cvs version: FreeBSD 6.1 (RELENG_6_1)" ;; 
			RELENG_6_0)
				echo " cvs version: FreeBSD 6.0 (RELENG_6_0)" ;; 
			RELENG_5)
				echo " cvs version: FreeBSD 5-STABLE (RELENG_5)" ;; 
			RELENG_5_5)
				echo " cvs version: FreeBSD 5.5 (RELENG_5_5)" ;; 
			RELENG_5_4)
				echo " cvs version: FreeBSD 5.4 (RELENG_5_4)" ;; 
			RELENG_5_3)
				echo " cvs version: FreeBSD 5.3 (RELENG_5_3)" ;; 
			RELENG_4)
				echo " cvs version: FreeBSD 4-STABLE (RELENG_4)" ;; 
			RELENG_4_11)
				echo " cvs version: FreeBSD 4.11 (RELENG_4_11)" ;; 
			RELENG_4_10)
				echo " cvs version: FreeBSD 4.10 (RELENG_4_10)" ;; 
			RELENG_4_9)
				echo " cvs version: FreeBSD 4.9 (RELENG_4_9)" ;; 
			.)
				echo " Updated to CURRENT version (HEAD)" ;;
			*)
				echo " Updated to cvs version $RELTAG";;
		esac
	else
		echo " Updated system source files: No"
	fi	# UPSRC
	
	if [ "$UPKERNEL" = "CVSUPONLY" ]
	then
		echo " No 'make buildworld' or 'make buildkernel' was performed."
	else
		if [ "$UPKERNEL" = "GENERIC" -o "$UPKERNEL" = "CUSTOM" ]
		then
			echo " A generic (standard) kernel was installed."
		fi
		if [ "$UPKERNCONF" != "" ]
		then
			echo " A custom kernel ($UPKERNCONF) was installed."
		fi
	fi # CVSUPONLY

	if [ "$UPCVSUPSERVER" = "automatic" -o "$UPCVSUPSERVER" = "fastest_cvsup" ]
	then	# used fastest_cvsup
		echo " fastest_cvsup was used to select the cvs server."
	else
		echo " The cvs server '$UPCVSUPSERVER' was accessed."
	fi
}

funcContinue () {
	if [ "$QUIETMODE" = "YES" ]
	then
		continueUpdate="YES"
	else
		echo -n "
Continue with these options? (y/n [y]): "
		read ans
		if [ "$ans" = "y" -o "$ans" = "Y" -o "$ans" = "" ]
		then # right to go, do nothing :]
			continueUpdate="YES"
		else
			# nope, user is not happy 
			continueUpdate="NO"
		fi
	fi # QUIETMODE
} # lastupdate

error () {
	# print out an error message based on an error code (errCode) then exit with the code
	case "$errCode" in
		2)
			errMsg="
$SCRIPTNAME has been designed to work with a FreeBSD system only." ;;
		4)
			errMsg="
$SCRIPTNAME requires cvsup to be installed. Try the following first:
 cd /usr/ports/net/cvsup-without-gui
 make clean ; make install clean
 rehash
 cd `pwd`" ;;
		8)
			errMsg="
The config file is incomplete or can not be read. The best way to fix
this is as follow:
 $SCRIPTNAME -C

If you still return this error, investigate the following file and 
delete it: 
 $PROFILECONF" ;;
		16)
			errMsg="
Unable to create a file on the system. Possible reasons are a full 
partition or a partition is marked read only. Either way $SCRIPTNAME can 
not continue." ;;
		32)
			errMsg="
CVSup encountered an error updating the ports. Can not continue like this.";;
		64)
			errMsg="
CVSup encounters an error updating the source files. Can not continue 
like this.";;
		128)
			errMsg="
Unable to update the ports index. Possible reasons are:
 * No network connection
 * Invalid permissions
";;
		256)
			errMsg="
Unable to compile the world using 'make buildworld'. Possible reasons 
are:
 * Invalid permissions
 * Full disk partition 
";;
		258)
			errMsg="
Unable to install the world binaries using 'make installworld'. 
Possible reasons are:
 * Invalid permissions
 * Full disk partition 
";;
		512)
			errMsg="
The kernel build failed. This could be for any number of reasons. Check
any error messages above and consult '/usr/src/UPDATING' before 
re-running $SCRIPTNAME
";;
		514)
			errMsg="
The kernel install failed. This could be for any number of reasons. 
Check any error messages above and consult '/usr/src/UPDATING' before 
re-running $SCRIPTNAME
";;

		1024)
			errMsg="
Unable to find the requested kernel config '$UPKERNCONF'.

Possible reasons for the error are:
 * A misspelling of the config name
 * Incorrectly located kernel config. The config should be located at:
   /usr/src/sys/$hardware/conf/$UPKERNCONF
 * Invalid permissions

The best way to fix this is to run the following:
 $SCRIPTNAME -C

Or you can edit the 'UPKERNCONF' value in:
 $PROFILECONF
";;
		2048)
			errMsg="
Problem running 'mergemaster'. You will need to run mergemaster 
manually. It is recommended that you check over /usr/src/UPDATING for 
possible required system changes before re-running $SCRIPTNAME.
";;
		4096)
			errMsg="
You have requested an option that requires the 'fastest_cvsup' script, 
however fastest_cvsup can not be found in your path. Please check that
fastest_cvsup is installed or run $SCRIPTNAME -C to clear the invalid
option.
";;
		8192)
			errMsg="
'fastest_cvsup' was unable to find any cvsup server. Best guesses are:
 * No network connection or a DNS issue
 * A mis-type on the TLD short list

The best way to fix this is to check your network connection. If your 
network is up and running, try running the following:
 $SCRIPTNAME -C
 $SCRIPTNAME
";;
	esac

	echo "
-----------------------------------------------------------------------
 Critical error (code: $errCode)
-----------------------------------------------------------------------
$errMsg
"
	exit $errCode
} # error

funcAppsCheck () {
	# make sure we have what we require before doing anything more
	# are we FreeBSD?
	if [ `uname` != "FreeBSD" ]
	then # not freebsd :[
		errCode="2"
		error
	fi 
	if [ "$CVSUPAPP" = "" ]
	then # cvsup is not installed :[
		errCode="4"
		error
	fi
	if [ ! -d "$HOME/.sysupdate" ]
	then	# maybe first time run, so create the storage folder
		mkdir "$HOME/.sysupdate"
	fi
} # funcAppsCheck

funcUserCanceled () {
	# User has canceled, so we let them know what has happened
	echo "
-----------------------------------------------------------------------
 A cancel request has been called.
-----------------------------------------------------------------------
"
	exit 1
	
} # funcUserCanceled

funcConfigPorts () {
	# sets the ports options

	/usr/bin/dialog --title "Update Options: Ports" --clear \
    --menu "\n\
Do you wish to update the ports collection?\n\n\
If you have installed the ports collection it is a good\n\
idea to update the 'Entire Collection'. If you are more\n\
experienced or know that you require only certain ports\n\
you can select the 'Partial Update' option. You will then\n\
be asked which ports to update. Press F1 or ? for more." -1 -1 3 \
"ports-all"	"Entire Collection" \
"custom"	"Partial Update" \
"none"	"Do NOT update the ports" \
2> $tmpOut

	if [ "$?" != 0 ]
	then    #user exit
		rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		tmp=`cat $tmpOut`
		echo "UPPORTS=\"$tmp\"" >> "$PROFILECONF"
		rm -f $tmpOut
		if [ "$tmp" = "custom" ]
		then	# need to run the custom ports option
			PORTSCUSTOM="1"
		fi
	fi # "$?"
} #funcConfigPorts

funcConfigPortsCustom () {
	# present the port section and allow a partial update of the ports
	# the long list is a bit of a pain... 
	/usr/bin/dialog --title "Update Options: Which Ports?" --clear \
	--hfile $PREFIX/files/hlp-ports \
	--checklist "\n\
You have asked for a custom ports update. You need to select\n\
which ports to update from the list below.\n\
Press F1 or ? for more information." -1 -1 13 \
"ports-base"	"Should be included" ON \
"ports-accessibility"	"Accessibility" OFF \
"ports-arabic"	"Arabic" OFF \
"ports-archivers"	"Archivers" OFF \
"ports-astro"	"Astro" OFF \
"ports-audio"	"Audio" OFF \
"ports-benchmarks"	"Benchmarks" OFF \
"ports-biology"	"Biology" OFF \
"ports-cad"	"CAD" OFF \
"ports-chinese"	"Chinese" OFF \
"ports-comms"	"Comms" OFF \
"ports-converters"	"Converters" OFF \
"ports-databases"	"Databases" OFF \
"ports-deskutils"	"Desk utils" OFF \
"ports-devel"	"Devel"	OFF \
"ports-dns"	"DNS" OFF \
"ports-editors"	"Editors" OFF \
"ports-emulators"	"Emulators" OFF \
"ports-finance"	"Finance" OFF \
"ports-french"	"French" OFF \
"ports-ftp"	"FTP" OFF \
"ports-games"	"Games" OFF \
"ports-german"	"German" OFF \
"ports-graphics"	"Graphics" OFF \
"ports-hebrew"	"Hebrew" OFF \
"ports-hungarian"	"Hungarian" OFF \
"ports-irc"	"IRC" OFF \
"ports-japanese"	"Japanese" OFF \
"ports-java"	"JAVA" OFF \
"ports-korean"	"Korean" OFF \
"ports-lang"	"Languages" OFF \
"ports-mail"	"Mail" OFF \
"ports-math"	"Math" OFF \
"ports-mbone"	"mbone" OFF \
"ports-misc"	"Miscellaneous" OFF \
"ports-multimedia"	"Multimedia" OFF \
"ports-net"	"Network related" OFF \
"ports-net-mgmt"	"Network management" OFF \
"ports-news"	"News" OFF \
"ports-palm"	"Palm / PDA related" OFF \
"ports-polish"	"Polish" OFF \
"ports-portuguese"	"Portuguese" OFF \
"ports-print"	"Print related" OFF \
"ports-russian"	"Russian" OFF \
"ports-science"	"Science" OFF \
"ports-security"	"Security" OFF \
"ports-shells"	"Shells" OFF \
"ports-sysutils"	"System utils" OFF \
"ports-textproc"	"Text processing" OFF \
"ports-ukrainian"	"Ukrainian" OFF \
"ports-vietnamese"	"Vietnamese" OFF \
"ports-www"	"Web related" OFF \
"ports-x11"	"X11 Base" OFF \
"ports-x11-clocks"	"X11 Clocks" OFF \
"ports-x11-fm"	"X11 File managers" OFF \
"ports-x11-fonts"	"X11 Fonts" OFF \
"ports-x11-servers"	"X11 Servers" OFF \
"ports-x11-themes"	"X11 Themes" OFF \
"ports-x11-toolkits"	"X11 Toolkits" OFF \
"ports-x11-wm"	"Window managers" OFF \
2> $tmpOut

	if [ "$?" != 0 ]
	then	# user canceled
		rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		# move the list to the correct location
		mv -f $tmpOut "$HOME/.sysupdate/portsCustom"
	fi
} # funcConfigPortsCustom

funcConfigPortsIndex () {
	# do they wish to update the their ports index?
	/usr/bin/dialog --title "Update Options: Ports" --clear \
    --menu "\n\
Do you wish to update the ports index?\n\n\
" -1 -1 3 \
yes	"Yes, run 'make fetchindex'." \
no	"No, do not update the index." \
2> $tmpOut

    if [ "$?" != 0 ]
    then    #user exit
        rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		tmp=`cat $tmpOut`
		echo "UPPORTSINDEX=\"$tmp\"" >> "$PROFILECONF"
        rm -f $tmpOut
	fi
} # funcConfigPortsIndex

funcConfigVersion () {
	# what FreeBSD version are they after?
	/usr/bin/dialog --title "Update Options: FreeBSD version" --clear \
	--menu "\n\
Select the FreeBSD version you wish to update to from the
list below. NOTE: It is NOT recommended that you use this
program to jump major builds (going from 5.5-STABLE to
6.2-RELEASE is a major build jump)." -1 -1 7 \
"custom"	"Enter a custom cvs tag" \
"none"	"Do NOT update core files. (Ports only)" \
"RELENG_6"	"RELENG_6 - FreeBSD 6-STABLE" \
"RELENG_6_2"	"RELENG_6_2 - FreeBSD 6.2" \
"RELENG_6_1"	"RELENG_6_1 - FreeBSD 6.1" \
"RELENG_5"	"RELENG_5 - FreeBSD 5-STABLE" \
"RELENG_5_5"	"RELENG_5_5 - FreeBSD 5.5" \
"RELENG_5_4"	"RELENG_5_4 - FreeBSD 5.4" \
"RELENG_4"	"RELENG_4 - FreeBSD 4-STABLE" \
"RELENG_4_11"	"RELENG_4_11 - FreeBSD 4.11" \
2> $tmpOut

    if [ "$?" != 0 ]
    then    #user exit
        rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		tmp=`cat $tmpOut`
		echo "UPRELTAG=\"$tmp\"" >> "$PROFILECONF"
        rm -f $tmpOut
		if [ "$tmp" = "none" ]
		then	# need to keep track of this info
			RELTAGNONE="1"
		fi
		if [ "$tmp" = "custom" ]
		then	# need to run the custom ports option
			RELTAGCUSTOM="1"
		fi
	fi # $?
} # funcConfigVersion

funcConfigVersionCustom () {
	# allow a user to enter a custom RELENG value
	 /usr/bin/dialog --title "Update Options: Custom cvs tag" --clear \
	--inputbox "\n\
You have chosen to enter a custom cvs tag.\n\
Enter the tag below (eg. RELENG_6):\n" 10 50 \
2> $tmpOut

	if [ "$?" != "0" ]
	then    # user quit 
		rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		# update the reltag var
		tmp=`cat $tmpOut`
		echo "UPRELTAGVERSION=\"$tmp\"" >> "$PROFILECONF"
		rm -f $tmpOut
	fi
} # funcConfigVersionCustom

funcConfigSrc () {
	# present the option to update the source files in full or not
	/usr/bin/dialog --title "Update Options: Which src files?" \
	--clear --menu "\n\
What core source files do you wish to update?\n\n\
It is recommended that you update the 'Entire Source'. If\n\
you are completely sure you do NOT need the entire core\n\
you can select the 'Partial Source' option, you will then\n\
be asked which sections to update." -1 -1 2 \
"src-all"	"Entire Source (recommended)" \
"custom"	"Partial Source" \
"none"	"Do not update the source files" \
2> $tmpOut

    if [ "$?" != 0 ]
    then    #user exit
        rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		tmp=`cat $tmpOut`
		echo "UPSOURCE=\"$tmp\"" >> "$PROFILECONF"
        rm -f $tmpOut
		if [ "$tmp" = "custom" ]
		then	# need to run the custom src option
			SOURCECUSTOM="1"
		fi
	fi # $?
} # funcConfigSrc

funcConfigSrcCustom () {
	# present a list of source file options to pick from
	/usr/bin/dialog --title "Update Options: Which src?" --clear \
	--checklist "\n\
You have asked for a partial update of the core source\n\
files. Select from the list below which files to\n\
update." -1 -1 13 \
"src-base" "base (recommended)" ON \
"src-bin" "bin" OFF \
"src-contrib" "contrib" OFF \
"src-crypto" "crypto" OFF \
"src-eBones" "eBones" OFF \
"src-etc" "etc" OFF \
"src-games" "games" OFF \
"src-gnu" "gnu" OFF \
"src-include" "include" OFF \
"src-kerberos5" "kerberos5" OFF \
"src-kerberosIV" "kerberosIV" OFF \
"src-lib" "lib" OFF \
"src-libexec" "libexec" OFF \
"src-release" "release" OFF \
"src-sbin" "sbin" OFF \
"src-secure" "secure" OFF \
"src-share" "share" OFF \
"src-sys" "sys" OFF \
"src-sys-crypto" "sys-crypto" OFF \
"src-tools" "tools" OFF \
"src-usrbin" "usrbin" OFF \
"src-usrsbin" "usrsbin" OFF \
2> $tmpOut

	if [ "$?" != 0 ]
	then	# user canceled
		rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		# move the list to the correct location
		mv -f $tmpOut "$HOME/.sysupdate/srcCustom"
	fi
} # funcConfigSrcCustom

funcConfigCvsUpServer () {
	# present a list of know cvsup server
	/usr/bin/dialog --title "Update Options: cvsup Server" --clear \
	--menu "\n\
Select a cvs server from the list below. The servers are ordered\n\
by countries. You can select a custom server be selecting\n\
the 'CUSTOM' option." -1 -1 10 \
"custom" "Enter your own details" \
"fastest_cvsup" "Requires fastest_cvsup" \
"automatic" "Requires fastest_cvsup" \
"cvsup.FreeBSD.org" "Central Servers" \
"cvsup1.FreeBSD.org" "Primary Mirror Sites" \
"cvsup2.FreeBSD.org" "Primary Mirror Sites" \
"cvsup3.FreeBSD.org" "Primary Mirror Sites" \
"cvsup4.FreeBSD.org" "Primary Mirror Sites" \
"cvsup5.FreeBSD.org" "Primary Mirror Sites" \
"cvsup6.FreeBSD.org" "Primary Mirror Sites" \
"cvsup7.FreeBSD.org" "Primary Mirror Sites" \
"cvsup8.FreeBSD.org" "Primary Mirror Sites" \
"cvsup9.FreeBSD.org" "Primary Mirror Sites" \
"cvsup10.FreeBSD.org" "Primary Mirror Sites" \
"cvsup11.FreeBSD.org" "Primary Mirror Sites" \
"cvsup12.FreeBSD.org" "Primary Mirror Sites" \
"cvsup13.FreeBSD.org" "Primary Mirror Sites" \
"cvsup14.FreeBSD.org" "Primary Mirror Sites" \
"cvsup15.FreeBSD.org" "Primary Mirror Sites" \
"cvsup16.FreeBSD.org" "Primary Mirror Sites" \
"cvsup18.FreeBSD.org" "Primary Mirror Sites" \
"cvsup.ar.FreeBSD.org" "Argentina" \
"cvsup.au.FreeBSD.org" "Australia" \
"cvsup2.au.FreeBSD.org" "Australia" \
"cvsup3.au.FreeBSD.org" "Australia" \
"cvsup4.au.FreeBSD.org" "Australia" \
"cvsup5.au.FreeBSD.org" "Australia" \
"cvsup6.au.FreeBSD.org" "Australia" \
"cvsup7.au.FreeBSD.org" "Australia" \
"cvsup.at.FreeBSD.org" "Austria" \
"cvsup2.at.FreeBSD.org" "Austria" \
"cvsup.br.FreeBSD.org" "Brazil" \
"cvsup2.br.FreeBSD.org" "Brazil" \
"cvsup3.br.FreeBSD.org" "Brazil" \
"cvsup4.br.FreeBSD.org" "Brazil" \
"cvsup5.br.FreeBSD.org" "Brazil" \
"cvsup.ca.FreeBSD.org" "Canada" \
"cvsup.cn.FreeBSD.org" "China" \
"cvsup2.cn.FreeBSD.org" "China" \
"cvsup3.cn.FreeBSD.org" "China" \
"cvsup4.cn.FreeBSD.org" "China" \
"cvsup5.cn.FreeBSD.org" "China" \
"cvsup1.cr.FreeBSD.org" "Costa Rica" \
"cvsup.cz.FreeBSD.org" "Czech Republic" \
"cvsup.dk.FreeBSD.org" "Denmark" \
"cvsup2.dk.FreeBSD.org" "Denmark" \
"cvsup3.dk.FreeBSD.org" "Denmark" \
"cvsup.ee.FreeBSD.org" "Estonia" \
"cvsup.fi.FreeBSD.org" "Finland" \
"cvsup2.fi.FreeBSD.org" "Finland" \
"cvsup.fr.FreeBSD.org" "France" \
"cvsup2.fr.FreeBSD.org" "France" \
"cvsup3.fr.FreeBSD.org" "France" \
"cvsup4.fr.FreeBSD.org" "France" \
"cvsup5.fr.FreeBSD.org" "France" \
"cvsup8.fr.FreeBSD.org" "France" \
"cvsup.de.FreeBSD.org" "Germany" \
"cvsup2.de.FreeBSD.org" "Germany" \
"cvsup3.de.FreeBSD.org" "Germany" \
"cvsup4.de.FreeBSD.org" "Germany" \
"cvsup5.de.FreeBSD.org" "Germany" \
"cvsup6.de.FreeBSD.org" "Germany" \
"cvsup7.de.FreeBSD.org" "Germany" \
"cvsup8.de.FreeBSD.org" "Germany" \
"cvsup.gr.FreeBSD.org" "Greece" \
"cvsup2.gr.FreeBSD.org" "Greece" \
"cvsup.hu.FreeBSD.org" "Hungary" \
"cvsup.is.FreeBSD.org" "Iceland" \
"cvsup.id.FreeBSD.org" "Indonesia" \
"cvsup.ie.FreeBSD.org" "Ireland" \
"cvsup.it.FreeBSD.org" "Italy" \
"cvsup.jp.FreeBSD.org" "Japan" \
"cvsup2.jp.FreeBSD.org" "Japan" \
"cvsup3.jp.FreeBSD.org" "Japan" \
"cvsup4.jp.FreeBSD.org" "Japan" \
"cvsup5.jp.FreeBSD.org" "Japan" \
"cvsup6.jp.FreeBSD.org" "Japan" \
"cvsup.kr.FreeBSD.org" "Korea" \
"cvsup2.kr.FreeBSD.org" "Korea" \
"cvsup3.kr.FreeBSD.org" "Korea" \
"cvsup1.kw.FreeBSD.org" "Kuwai" \
"cvsup.kg.FreeBSD.org" "Kyrgyzstan" \
"cvsup.lv.FreeBSD.org" "Latvia" \
"cvsup.lt.FreeBSD.org" "Lithuania" \
"cvsup2.lt.FreeBSD.org" "Lithuania" \
"cvsup3.lt.FreeBSD.org" "Lithuania" \
"cvsup.nl.FreeBSD.org" "Netherlands" \
"cvsup2.nl.FreeBSD.org" "Netherlands" \
"cvsup3.nl.FreeBSD.org" "Netherlands" \
"cvsup5.nl.FreeBSD.org" "Netherlands" \
"cvsup.nz.FreeBSD.org" "New Zealand" \
"cvsup.no.FreeBSD.org" "Norway" \
"cvsup1.ph.FreeBSD.org" "Philippines" \
"cvsup.pl.FreeBSD.org" "Poland" \
"cvsup2.pl.FreeBSD.org" "Poland" \
"cvsup3.pl.FreeBSD.org" "Poland" \
"cvsup.pt.FreeBSD.org" "Portugal" \
"cvsup2.pt.FreeBSD.org" "Portugal" \
"cvsup3.pt.FreeBSD.org" "Portugal" \
"cvsup.ro.FreeBSD.org" "Romania" \
"cvsup1.ro.FreeBSD.org" "Romania" \
"cvsup2.ro.FreeBSD.org" "Romania" \
"cvsup3.ro.FreeBSD.org" "Romania" \
"cvsup.ru.FreeBSD.org" "Russia" \
"cvsup2.ru.FreeBSD.org" "Russia" \
"cvsup3.ru.FreeBSD.org" "Russia" \
"cvsup4.ru.FreeBSD.org" "Russia" \
"cvsup5.ru.FreeBSD.org" "Russia" \
"cvsup6.ru.FreeBSD.org" "Russia" \
"cvsup.sm.FreeBSD.org" "San Marino" \
"cvsup.sg.FreeBSD.org" "Singapore" \
"cvsup.sk.FreeBSD.org" "Slovak Republic" \
"cvsup2.sk.FreeBSD.org" "Slovak Republic" \
"cvsup.si.FreeBSD.org" "Slovenia" \
"cvsup2.si.FreeBSD.org" "Slovenia" \
"cvsup.za.FreeBSD.org" "South Africa" \
"cvsup2.za.FreeBSD.org" "South Africa" \
"cvsup.es.FreeBSD.org" "Spain" \
"cvsup2.es.FreeBSD.org" "Spain" \
"cvsup3.es.FreeBSD.org" "Spain" \
"cvsup.se.FreeBSD.org" "Sweden" \
"cvsup3.se.FreeBSD.org" "Sweden" \
"cvsup.ch.FreeBSD.org" "Switzerland" \
"cvsup.tw.FreeBSD.org" "Taiwan" \
"cvsup3.tw.FreeBSD.org" "Taiwan" \
"cvsup4.tw.FreeBSD.org" "Taiwan" \
"cvsup5.tw.FreeBSD.org" "Taiwan" \
"cvsup6.tw.FreeBSD.org" "Taiwan" \
"cvsup7.tw.FreeBSD.org" "Taiwan" \
"cvsup8.tw.FreeBSD.org" "Taiwan" \
"cvsup9.tw.FreeBSD.org" "Taiwan" \
"cvsup10.tw.FreeBSD.org" "Taiwan" \
"cvsup11.tw.FreeBSD.org" "Taiwan" \
"cvsup12.tw.FreeBSD.org" "Taiwan" \
"cvsup13.tw.FreeBSD.org" "Taiwan" \
"cvsup.th.FreeBSD.org" "Thailand" \
"cvsup.tr.FreeBSD.org" "Turkey" \
"cvsup2.ua.FreeBSD.org" "Ukraine" \
"cvsup3.ua.FreeBSD.org" "Ukraine" \
"cvsup4.ua.FreeBSD.org" "Ukraine" \
"cvsup5.ua.FreeBSD.org" "Ukraine" \
"cvsup6.ua.FreeBSD.org" "Ukraine" \
"cvsup7.ua.FreeBSD.org" "Ukraine" \
"cvsup.uk.FreeBSD.org" "United Kingdom" \
"cvsup2.uk.FreeBSD.org" "United Kingdom" \
"cvsup3.uk.FreeBSD.org" "United Kingdom" \
"cvsup4.uk.FreeBSD.org" "United Kingdom" \
"cvsup1.us.FreeBSD.org" "USA" \
"cvsup2.us.FreeBSD.org" "USA" \
"cvsup3.us.FreeBSD.org" "USA" \
"cvsup4.us.FreeBSD.org" "USA" \
"cvsup5.us.FreeBSD.org" "USA" \
"cvsup6.us.FreeBSD.org" "USA" \
"cvsup7.us.FreeBSD.org" "USA" \
"cvsup8.us.FreeBSD.org" "USA" \
"cvsup9.us.FreeBSD.org" "USA" \
"cvsup10.us.FreeBSD.org" "USA" \
"cvsup11.us.FreeBSD.org" "USA" \
"cvsup12.us.FreeBSD.org" "USA" \
"cvsup13.us.FreeBSD.org" "USA" \
"cvsup14.us.FreeBSD.org" "USA" \
"cvsup15.us.FreeBSD.org" "USA" \
"cvsup16.us.FreeBSD.org" "USA" \
"cvsup18.us.FreeBSD.org" "USA" \
2> $tmpOut

	if [ "$?" != 0 ]
	then    #user exit
		rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		tmp=`cat $tmpOut`
		echo "UPCVSUPSERVER=\"$tmp\"" >> "$PROFILECONF"
		rm -f $tmpOut
		if [ "$tmp" = "custom" ]
		then	# need to run the custom src option
			CVSUPSERVERCUSTOM="1"
		fi
		if [ "$tmp" = "fastest_cvsup" ]
		then	# need to run the fastest_cvs option
			CVSUPSERVERFASTEST="1"
		fi
		if [ "$tmp" = "automatic" ]
		then	# need to run the fastest_cvs option
			CVSUPSERVERFASTEST="1"
		fi
	fi # "$?"

} # funcConfigCvsUpServer

funcConfigCvsUpServerCustom () {
	# present an input box for a cvsup server
	/usr/bin/dialog --title "Update Options: Custom CVSup server" --clear \
	--inputbox "\n\
You have chosen to enter a custom cvs server.\n\
Enter the address of your cvs server below:\n\
(eg. cvs.mydomain.net)" 10 50 \
2> $tmpOut

	if [ "$?" != "0" ]
	then    # user quit 
		rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		# record the cvsup server URI
		tmp=`cat $tmpOut`
		echo "UPCVSUPSERVERCUSTOM=\"$tmp\"" >> "$PROFILECONF"
		tmpCvsUpServer=$tmp
		rm -f $tmpOut
	fi

} # funcConfigCvsUpServerCustom

funcConfigCvsUpServerFastest () {
	# present an input box for a TLD list
	/usr/bin/dialog --title "Update Options: Fastest CVS TLD options" --clear \
	--inputbox "\n\
You have chosen to use the fastest_cvs tool\n\
to locate a cvs server. By default all servers\n\
all servers are used, however you can enter a\n\
comma list of TLD to speed-up the process. For\n\
(eg. au,nz,jp,us)" 12 54 \
2> $tmpOut

	if [ "$?" != "0" ]
	then    # user quit 
		rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		# record the cvsup server URI
		tmp=`cat $tmpOut`
		if [ "$tmp" = "" ]
		then # default to 'all'
			tmp="all"
		fi
			echo "UPCVSUPSERVERFASTEST=\"$tmp\"" >> "$PROFILECONF"
		rm -f $tmpOut
	fi
} # funcConfigCvsUpServerFastest

funcConfigKernel () {
	# present the kernel and build options 
	# NOTE: while it is possible to do a make buildworld without a 
	# make buildkernel, it is not a good idea
	 # the default message
	/usr/bin/dialog --title "Update Options: Build Options" \
	--clear --menu "\n\
Below are a list of build options relating to the updating
of your system and kernel. Unless the 'CVSup only' option is 
selected a 'make buildworld' will be performed, followed by
the kernel compile option from the list below.

If you select to install a CUSTOM kernel, you will be asked 
for the name of the kernel." -1 -1 4 \
"GENERIC"	"GENERIC kernel only" \
"CUSTOM"	"Install a GENERIC kernel then a CUSTOM kernel" \
"CUSTOMONLY"	"Install a CUSTOM kernel only (NOT recommended)" \
"CVSUPONLY"	"CVSup only (no buildworld or buildkernel)" \
2> $tmpOut

	if [ "$?" != 0 ]
	then    #user exit
		rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		tmp=`cat $tmpOut`
		echo "UPKERNEL=\"$tmp\"" >> "$PROFILECONF"
		rm -f $tmpOut
		if [ "$tmp" = "CUSTOM" -o "$tmp" = "CUSTOMONLY" ]
		then	# need to run the custom kernel option
			KERNELCUSTOM="1"
		fi
	fi # "$?"
} # funcConfigKernel

funcConfigKernelCustom () {
	# what is the name of the custom kernel
	tmpK=`uname -i`
	/usr/bin/dialog --title "Update Options: Custom Kernel Name" --clear \
	--inputbox "\n\
You have chosen to install a custom kernel.\n\
Enter the name of your kernel config below:\n\
(eg. $tmpK)" 10 50 \
2> $tmpOut

	if [ "$?" != "0" ]
	then    # user quit 
		rm -f $tmpOut
		rm -f "$PROFILECONF"
		funcUserCanceled
	else
		# take note of the kernconf 
		tmp=`cat $tmpOut`
		echo "UPKERNCONF=\"$tmp\"" >> "$PROFILECONF"
		rm -f $tmpOut
	fi
} # funcConfigKernelCustom

funcConfig () {
	# step through the options of updating a system
	
	# first delete the current config
	echo "# $SCRIPTNAME config - created `date`" > "$PROFILECONF"
	if [ ! -s "$PROFILECONF" ]
	then	# we have a problem - we need to be able to write to a file
		errCode=16
		error
	fi

	# need to clear the lastupdate file as this is a clean config
	rm -f "$HOME/.sysupdate/lastupdate"

	# make a temp file
    tmpOut=`mktemp /tmp/XXXXXXXX`
	if [ ! -O $tmpOut ]
	then	# we have a problem - we need to be able to write to a file
		errCode=16
		error
	fi

	# What port options do they want?
	funcConfigPorts

	if [ "$PORTSCUSTOM" = "1" ]
	then # They want a custom selection of ports
		funcConfigPortsCustom
	fi

	# offer a 'make fetchindex' option
	funcConfigPortsIndex

	# what source file option do they want?
	funcConfigVersion

	if [ "$RELTAGCUSTOM" = "1" ]
	then # they want to enter a custom tag
		funcConfigVersionCustom
	fi

	if [ "$RELTAGNONE" != "1" ]
	then
		# offer a custom source file option (with advice against it)
		funcConfigSrc
	fi

	if [ "$SOURCECUSTOM" = "1" ]
	then # they want to enter a custom list 
		funcConfigSrcCustom
	fi

	# generic kernel compile or custom?
	funcConfigKernel 
	# custom kernel
	if [ "$KERNELCUSTOM" = "1" ]
	then # they want to enter a custom cvsup server
		funcConfigKernelCustom
	fi
	
	# What cvsup server to use?
	funcConfigCvsUpServer

	if [ "$CVSUPSERVERCUSTOM" = "1" ]
	then # they want to enter a custom cvsup server
		funcConfigCvsUpServerCustom
	fi

	if [ "$CVSUPSERVERFASTEST" = "1" ]
	then # they want to use fastest_cvs to find a server
		funcConfigCvsUpServerFastest
	fi

	if [ -s "$HOME/.sysupdate/status" ]
	then #remove it so we have a clean status
		rm -f "$HOME/.sysupdate/status"
	fi

} # funcConfig

funcCVSPorts () {
	# update the ports
	
	if [ "$UPPORTS" = "none" ]
	then # they are not updating the ports, so return for the next step
		LASTACTION="cvsupPorts"
		echo "LASTACTION=\"cvsupPorts\"" > "$HOME/.sysupdate/status"
		return
	fi

	# write the cvsup ports config
	echo "
*default host=$UPCVSUPSERVER
*default base=/var/db
*default prefix=/usr
*default release=cvs tag=.
*default delete use-rel-suffix

*default compress
" > "$HOME/.sysupdate/cvsupPorts"

	if [ "$UPPORTS" = "custom" ]
	then # we need to pull in the custom list
		tmp=`cat "$HOME/.sysupdate/portsCustom" | sed 's/\"//g'`
		for p in $tmp
		do
			echo $p >> "$HOME/.sysupdate/cvsupPorts"
			#PORTLIST="$PORTLIST $p"
		done
		#echo $PORTLIST
	fi
	if [ "$UPPORTS" = "ports-all" ]
	then # default ports update
		echo "ports-all" >> "$HOME/.sysupdate/cvsupPorts" 
	fi

	if [ "$UPPORTS" = "custom" -o "$UPPORTS" = "ports-all" ]
	then # cool, run the ports update
		echo "INFO: Starting the CVSup of the ports: `date`"

		if [ "$DEBUG" = "YES" ]
		then # just print the action
			echo "DEBUG: $CVSUPAPP -g -L 2 \"$HOME/.sysupdate/cvsupPorts\""
		else 
			$CVSUPAPP -g -L 2 "$HOME/.sysupdate/cvsupPorts"
		fi
		# check for any error
		if [ "$?" != "0" ]
		then # there was a problem
			errCode="32"
			error
		else
			# all is good, keep moving
			LASTACTION="cvsupPorts"
			echo "LASTACTION=\"cvsupPorts\"" > "$HOME/.sysupdate/status"
			return
		fi # error or not
	fi # custom or ports-all
} # funcCVSPorts

funcCVSSource () {
	# update the source 
	if [ "$UPRELTAG" = "none" ]
	then # they are not updating the ports, so return for the next step
		LASTACTION="cvsupSource"
		echo "LASTACTION=\"cvsupSource\"" > "$HOME/.sysupdate/status"
		return
	fi

	if [ "$UPRELTAG" = "custom" ]
	then # they are after a custom version, so make sure they get it
		tmpRelTagVersion=$UPRELTAGVERSION
	else
		# they get what they asked for too
		tmpRelTagVersion=$UPRELTAG
	fi # UPRELTAG custom?

	# write the cvsup source config
	echo "
*default host=$UPCVSUPSERVER
*default base=/var/db
*default prefix=/usr
*default release=cvs tag=$tmpRelTagVersion
*default delete use-rel-suffix

*default compress
" > "$HOME/.sysupdate/cvsupSource"

	if [ "$UPSOURCE" = "custom" ]
	then # we need to pull in the custom list
		tmp=`cat "$HOME/.sysupdate/srcCustom" | sed 's/\"//g'`
		for p in $tmp
		do
			echo $p >> "$HOME/.sysupdate/cvsupSource"
		done
	fi

	if [ "$UPSOURCE" = "src-all" ]
	then # default source update
		echo "src-all" >> "$HOME/.sysupdate/cvsupSource" 
	fi

	if [ "$UPSOURCE" = "custom" -o "$UPSOURCE" = "src-all" ]
	then # cool, run the cvsup update
		echo "INFO: Starting the CVSup of the source files: `date`"
	
		if [ "$DEBUG" = "YES" ]
		then # just print the action
			echo "DEBUG: $CVSUPAPP -g -L 2 \"$HOME/.sysupdate/cvsupSource\""
		else
			$CVSUPAPP -g -L 2 "$HOME/.sysupdate/cvsupSource"
		fi
		# check for any error
		if [ "$?" != "0" ]
		then # there was a problem
			errCode="64"
			error
		else
			# all is good, keep moving
			LASTACTION="cvsupSource"
			echo "LASTACTION=\"cvsupSource\"" > "$HOME/.sysupdate/status"
			return
		fi # error or not
	fi # custom or ports-all
} # funcCVSSource

funcFetchIndex () {
	# updates the ports index
	if [ "$UPPORTSINDEX" != "yes" ]
	then # return for the next step 
		LASTACTION="fetchIndex"
		echo "LASTACTION=\"fetchIndex\"" > "$HOME/.sysupdate/status"
		return
	fi
	
	# if we are here, assume we are updating
	# keep track of where we are
	thisLocation="`pwd`"
	echo "INFO: Updating the ports index : `date`"
	if [ "$DEBUG" = "YES" ]
	then # just print the action
		echo "DEBUG: cd /usr/ports"
		echo "DEBUG: make fetchindex"
	else
		cd /usr/ports
		make fetchindex	
	fi	

	if [ "$?" != "0" ]
	then # problem with the make fetchindex?
		errCode="128"
		error
	fi	

	# no error, return to previous location
	cd "$thisLocation"
	LASTACTION="fetchIndex"
	echo "LASTACTION=\"fetchIndex\"" > "$HOME/.sysupdate/status"
	return		
} # funcFetchIndex

funcViewUPDATING () {
	# display the UPDATING file before we do anything else (if we are updating that is)
	if [ "$UPKERNEL" = "CVSUPONLY" ]
	then # return - nothing left to do! 
		LASTACTION="finished"
		echo "LASTACTION=\"finished\"" > "$HOME/.sysupdate/status"
		return
	fi

	if [ -s "/usr/src/UPDATING" ]
	then # read the file!
		if [ "$QUIETMODE" != "YES" ]
		then # don't display in quiet mode
			echo "-----------------------------------------------------------------------
Information about updating your system is below. Of significant 
interest are any entries with a date between now and your last update.

If required, you may need to make changes before continuing.

Use the up and down arrows or J and K to scroll through the information.
Press Q to quit the reader.
-----------------------------------------------------------------------
" > "$HOME/.sysupdate/UPDATING"
			cat /usr/src/UPDATING >> "$HOME/.sysupdate/UPDATING"
			less "$HOME/.sysupdate/UPDATING"

			# do they wish to continue or exit?
			echo -n "
Continue with update? (y/n [y]): "
			read ans
			if [ "$ans" = "y" -o "$ans" = "Y" -o "$ans" = "" ]
			then # cool, move onto the next step
				LASTACTION="viewUpdate"
				echo "LASTACTION=\"viewUpdate\"" > "$HOME/.sysupdate/status"
			else
				# they want to bail at this point
				echo "-----------------------------------------------------------------------
 Exiting after reading the UPDATING information
-----------------------------------------------------------------------
"
				exit 1
			fi # ans
		else 
			# in quiet mode
			LASTACTION="viewUpdate"
			echo "LASTACTION=\"viewUpdate\"" > "$HOME/.sysupdate/status"
		fi # QUIETMODE
	else 
		# this file should not be missing, but if it is, is it a critical error?
		echo "
PROBLEM: The /usr/src/UPDATING file is missing! There could be a 
         problem with your update but continuing anyway."
		sleep 10
		LASTACTION="viewUpdate"
		echo "LASTACTION=\"viewUpdate\"" > "$HOME/.sysupdate/status"
	fi # UPDATING found or not
} # funcViewUPDATING

funcBuildWorld () {
	# build the world binaries 
	if [ "$UPKERNEL" = "CVSUPONLY" ]
	then # return - nothing left to do! 
		LASTACTION="finished"
		echo "LASTACTION=\"finished\"" > "$HOME/.sysupdate/status"
		return
	fi
	
	# if we are here, assume we are updating
	# keep track of where we are
	thisLocation="`pwd`"
	echo "INFO: Compiling the system binaries (make buildworld): `date`"
	# remove any /usr/obj files
	if [ -d "/usr/obj/usr" ]
	then
		echo "INFO: Removing old build files: `date`"
		if [ "$DEBUG" = "YES" ]
		then
			echo "DEBUG: rm -rf /usr/obj/usr"
		else
			rm -rf /usr/obj/usr
		fi
	fi # obj/usr check

	if [ "$JVALUE" = "" ]
	then # assume no value has been requested for -j 
		if [ "$DEBUG" = "YES" ]
		then
			echo "DEBUG: cd /usr/src"
			echo "DEBUG: make buildworld"
		else
			cd /usr/src
			make buildworld 
		fi # debug
	else
		if [ "$DEBUG" = "YES" ]
		then
			echo "DEBUG: cd /usr/src"
			echo "DEBUG: make -j$JVALUE buildworld"
		else
			cd /usr/src
			make -j$JVALUE buildworld
		fi # debug
	fi # buildworld
	
	if [ "$?" != "0" ]
	then # problem with the make ?
		errCode="256"
		error
	fi	

	# no error, return to previous location
	cd "$thisLocation"
	LASTACTION="buildworld"
	echo "LASTACTION=\"buildworld\"" > "$HOME/.sysupdate/status"
	return
} # funcBuildWorld

funcBuildKernel () {
	# build a generic kernel
	if [ "$UPKERNEL" = "CUSTOMONLY" ]
	then # return - nothing left to do here! 
		LASTACTION="buildkernel"
		echo "LASTACTION=\"buildkernel\"" > "$HOME/.sysupdate/status"
		return
	fi
	
	# if we are here, assume we are updating
	# keep track of where we are
	thisLocation="`pwd`"
	echo "INFO: Compiling a GENERIC kernel (make buildkernel): `date`"
	if [ "$DEBUG" = "YES" ]
	then
		echo "DEBUG: cd /usr/src"
		echo "DEBUG: make buildkernel"
	else
		cd /usr/src
		make buildkernel
	fi # debug
	
	if [ "$?" != "0" ]
	then # problem with the make ?
		errCode="512"
		error
	fi	

	# no error, return to previous location
	cd "$thisLocation"
	LASTACTION="buildkernel"
	echo "LASTACTION=\"buildkernel\"" > "$HOME/.sysupdate/status"
	return
} # funcBuildKernel

funcBuildKernelCustom () {
	# build a custom kernel
	if [ "$UPKERNEL" = "GENERIC" ]
	then # return - nothing left to do here! 
		LASTACTION="buildkernelCustom"
		echo "LASTACTION=\"buildkernelCustom\"" > "$HOME/.sysupdate/status"
		return
	fi
	
	# make sure we have what we need
	if [ "$UPKERNCONF" = "" ]
	then # this should NOT happen
		errCode="8"
		error
	fi		

	hardware=`uname -m`

	if [ ! -s "/usr/src/sys/$hardware/conf/$UPKERNCONF" ]
	then # they have requested a kernel config that does not exist :[
		errCode="1024"
		error
	fi

	# if we are here, assume we are updating
	# keep track of where we are
	thisLocation="`pwd`"
	echo "INFO: Compiling a custom kernel ($UPKERNCONF): `date`"
	if [ "$DEBUG" = "YES" ]
	then
		echo "DEBUG: cd /usr/src"
		echo "DEBUG: make buildkernel KERNCONF=$UPKERNCONF"
	else
		cd /usr/src
		make buildkernel KERNCONF=$UPKERNCONF
	fi # debug
	
	if [ "$?" != "0" ]
	then # problem with the make ?
		errCode="512"
		error
	fi	

	# no error, return to previous location
	cd "$thisLocation"
	LASTACTION="buildkernelCustom"
	echo "LASTACTION=\"buildkernelCustom\"" > "$HOME/.sysupdate/status"
	return
} # funcBuildKernelCustom

funcInstallKernel () {
	# install a generic kernel
	if [ "$UPKERNEL" = "CUSTOMONLY" ]
	then # return - nothing left to do here! 
		LASTACTION="installkernel"
		echo "LASTACTION=\"installkernel\"" > "$HOME/.sysupdate/status"
		return
	fi
	
	# keep track of where we are
	thisLocation="`pwd`"
	# backup the current kernel
	echo "INFO: Backing up the current kernel to kernel.sysupdate: `date`"
	if [ "$DEBUG" = "YES" ]
	then
		echo "DEBUG: cp -Rpf /boot/kernel /boot/kernel.sysupdate"
	else
		cp -Rpf /boot/kernel /boot/kernel.sysupdate
	fi

	echo "INFO: Installing a GENERIC kernel (make installkernel): `date`"
	if [ "$DEBUG" = "YES" ]
	then
		echo "DEBUG: cd /usr/src"
		echo "DEBUG: make installkernel"
	else
		cd /usr/src
		make installkernel
	fi # debug
	
	if [ "$?" != "0" ]
	then # problem with the make ?
		errCode="514"
		error
	fi	

	# no error, return to previous location
	cd "$thisLocation"
	LASTACTION="installkernel"
	echo "LASTACTION=\"installkernel\"" > "$HOME/.sysupdate/status"
	return
} # funcInstallKernel

funcInstallKernelCustom () {
	# install a custom kernel
	if [ "$UPKERNEL" = "GENERIC" ]
	then # return - nothing left to do here! 
		LASTACTION="installkernelCustom"
		echo "LASTACTION=\"installkernelCustom\"" > "$HOME/.sysupdate/status"
		return
	fi

	if [ "$UPKERNEL" = "CUSTOMONLY" ]
	then
		# backup the current kernel
		echo "INFO: Backing up the current kernel to kernel.sysupdate: `date`"
		if [ "$DEBUG" = "YES" ]
		then
			echo "DEBUG: cp -Rpf /boot/kernel /boot/kernel.sysupdate"
		else
			cp -Rpf /boot/kernel /boot/kernel.sysupdate
		fi
	fi
		
	# if we are here, assume we are updating
	# keep track of where we are
	thisLocation="`pwd`"
	echo "INFO: Installing a custom kernel ($UPKERNCONF): `date`"
	if [ "$DEBUG" = "YES" ]
	then
		echo "DEBUG: cd /usr/src"
		echo "DEBUG: make installkernel KERNCONF=$UPKERNCONF"
	else
		cd /usr/src
		make installkernel KERNCONF=$UPKERNCONF
	fi # debug
	
	if [ "$?" != "0" ]
	then # problem with the make ?
		errCode="512"
		error
	fi	

	# no error, return to previous location
	cd "$thisLocation"
	LASTACTION="installkernelCustom"
	echo "LASTACTION=\"installkernelCustom\"" > "$HOME/.sysupdate/status"
	return
} # funcInstallKernelCustom

funcInstallWorld () {
	# install the world binaries 
	if [ "$UPKERNEL" = "CVSUPONLY" ]
	then # return - nothing left to do! 
		LASTACTION="finished"
		echo "LASTACTION=\"finished\"" > "$HOME/.sysupdate/status"
		return
	fi
	
	# keep track of where we are
	thisLocation="`pwd`"
	echo "INFO: Installing the system binaries (make installworld): `date`"
	if [ "$JVALUE" = "" ]
	then # assume no value has been requested for -j 
		if [ "$DEBUG" = "YES" ]
		then
			echo "DEBUG: cd /usr/src"
			echo "DEBUG: make installworld"
		else
			cd /usr/src
			make installworld 
		fi # debug
	else
		if [ "$DEBUG" = "YES" ]
		then
			echo "DEBUG: cd /usr/src"
			echo "DEBUG: make -j$JVALUE installworld"
		else
			cd /usr/src
			make -j$JVALUE installworld
		fi # debug
	fi # buildworld
	
	if [ "$?" != "0" ]
	then # problem with the make ?
		errCode="258"
		error
	fi	

	# no error, return to previous location
	cd "$thisLocation"
	LASTACTION="installworld"
	echo "LASTACTION=\"installworld\"" > "$HOME/.sysupdate/status"
	return
} # funcInstallWorld

funcMergemasterP () {
	# run mergemaster -p
	if [ "$UPKERNEL" = "CVSUPONLY" ]
	then # return - nothing left to do! 
		# NOTE: Should NOT get to here if cvsuponly, however we check anyway
		LASTACTION="mergemasterP"
		echo "LASTACTION=\"mergemasterP\"" > "$HOME/.sysupdate/status"
		return
	fi

	if [ "$QUIETMODE" = "YES" ]
	then # do not run this option
		echo "INFO: In quiet mode, we skip the mergemaster -p step: `date`"
		return
	fi
	
	echo "INFO: Backing up /etc to /etc-sysupdate1: `date`"
	if [ "$DEBUG" = "YES" ]
	then
		echo "DEBUG: cp -Rpf /etc /etc-sysupdate1"
	else 
		cp -Rpf /etc /etc-sysupdate1
	fi	
	
	if [ "$?" != "0" ]
	then # problem with backup? Is this critical?
		echo "-----------------------------------------------------------------------
Problem making a full backup of /etc. While this is not critical, it
is best to check any output above. The update will continue anyway.
-----------------------------------------------------------------------
"
		sleep 10
	fi

	echo "INFO: Running mergemaster -p: `date`"
	if [ "$DEBUG" = "YES" ]
	then
		echo "DEBUG: mergemaster -p"
	else
		mergemaster -p
	fi # debug

	if [ "$?" != "0" ]
	then # problem with mergemaster
		errCode="2048"
		error
	fi	

	# no error, return to previous location
	cd "$thisLocation"
	LASTACTION="mergemasterP"
	echo "LASTACTION=\"mergemasterP\"" > "$HOME/.sysupdate/status"
	return
} # funcMergemasterP

funcMergemaster () {
	# run mergemaster 
	if [ "$UPKERNEL" = "CVSUPONLY" ]
	then # return - nothing left to do! 
		# NOTE: Should NOT get to here if cvsuponly, however we check anyway
		LASTACTION="finished"
		echo "LASTACTION=\"finished\"" > "$HOME/.sysupdate/status"
		return
	fi

	echo "INFO: Backing up /etc to /etc-sysupdate: `date`"
	if [ "$DEBUG" = "YES" ]
	then
		echo "DEBUG: cp -Rpf /etc /etc-sysupdate"
	else 
		cp -Rpf /etc /etc-sysupdate
	fi	
	
	if [ "$?" != "0" ]
	then # problem with backup? Is this critical?
		echo "-----------------------------------------------------------------------
Problem making a full backup of /etc. While this is not critical, it
is best to check any output above. The update will continue anyway.
-----------------------------------------------------------------------
"
		sleep 10
	fi

	if [ "$QUIETMODE" = "YES" ]
	then # run mergemaster with the -ai flage
		echo "INFO: Running mergemaster -ai -t: `date`"
		tmpMergemasterRoot=`date +%Y%m%d%H%M`
		echo "INFO: Removing any old sysupdate mergemaster roots"
		rm -rf /tmp/sysupdate-merge*
		if [ "$DEBUG" = "YES" ]
		then
			echo "DEBUG: mergemaster -ai -t /tmp/sysupdate-merge$tmpMergemasterRoot"
		else
			mergemaster -ai -t /tmp/sysupdate-merge$tmpMergemasterRoot
		fi # debug

	else
		echo "INFO: Running mergemaster: `date`"
		if [ "$DEBUG" = "YES" ]
		then
			echo "DEBUG: mergemaster"
		else
			mergemaster
		fi # debug
	fi	

	if [ "$?" != "0" ]
	then # problem with mergemaster
		errCode="2048"
		error
	fi	

	# no error, return to previous location
	cd "$thisLocation"
	LASTACTION="mergemaster"
	echo "LASTACTION=\"mergemaster\"" > "$HOME/.sysupdate/status"
	return
} # funcMergemaster

funcRebootOption () {
	# reboot option after running mergemaster
	echo "-----------------------------------------------------------------------
 Important information
-----------------------------------------------------------------------
"
	if [ "$QUIETMODE" = "YES" ]
	then # we do NOT reboot in quite mode
		echo "Your system has been updated and mergemaster has been run 
with the following command line:
 mergemaster -ai -t /tmp/sysupdate-merge$tmpMergemasterRoot

Which means you should check the tmp folder for any files that will 
need to be compared to your current files before rebooting.

A reboot is required after you have checked the mergemaster files.

Alternatively, you could re-run mergemaster, then reboot.
"
		LASTACTION="finished"		
		#echo "LASTACTION=\"finished\"" > "$HOME/.sysupdate/status"		
		funcWriteLastUpdate

	else
		# we offer the option to reboot later
		echo "Your system has been updated and a reboot is highly recommended."
		echo -n "Do you want to reboot now? (y/n [y]) "
		read ans
		if [ "$ans" = "y" -o "$ans" = "Y" -o "$ans" = "" ]
		then # time to reboot
			#echo "LASTACTION=\"finished\"" > "$HOME/.sysupdate/status"		
			funcWriteLastUpdate
			# no need to print anything because we are rebooting
			if [ "$DEBUG" = "YES" ]
			then # only print the action
				echo "DEBUG: shutdown -r now"
				exit
			else
				shutdown -r now
				exit
			fi
		
		else
			# reboot required!
			LASTACTION="rebootRequired"
			echo "LASTACTION=\"rebootRequired\"" > "$HOME/.sysupdate/status"
			return
		fi # ans = y
	fi # QUIETMODE

} # funcRebootOption

funcWriteLastUpdate () {
	# writes out the 'lastupdate' info
	echo "# sysupdate.sh information file - created `date`
LASTUPDATE=\"`date`\"
LASTPROFILECONF=\"$PROFILECONF\"" > "$HOME/.sysupdate/lastupdate"
	# NOTE: Should really check for that this file is written
	# Now remove the status file
	rm -f $HOME/.sysupdate/status
} # funcWriteLastUpdate

funcRebootRequired () {
	# inform the user, then cleans up for next script run
	echo "-----------------------------------------------------------------------
 Important information
-----------------------------------------------------------------------

Make sure you have restarted the system before running $SCRIPTNAME
again.
"
	LASTACTION="finished"
	return
} # funcRebootRequired

funcFinished () {
	# exits the script, with a little message
	echo "-----------------------------------------------------------------------
 System update has finished : `date`
-----------------------------------------------------------------------"
	exit 0

} # funcFinished

funcMain () {
	# called after each step and based on the value of LASTACTION
	# a 'step' in the update process is performed

	# make sure we have the correct variables
	if [ -s "$PROFILECONF" ]
	then
		. "$PROFILECONF"
	fi

	# confirm we have everything we need before moving forward
	if [ "$UPPORTS" = "" ]
	then # can't be valid
		errCode="8"
		error
	fi
	if [ "$UPRELTAG" = "" ]
	then # can't be valid
		errCode="8"
		error
	fi
	if [ "$UPCVSUPSERVER" = "" ]
	then # can't be valid
		errCode="8"
		error
	fi
	if [ "$UPKERNEL" = "" ]
	then # can't be valid
		errCode="8"
		error
	fi

	# perform a few CVSup server task if required
	if [ "$UPCVSUPSERVER" = "custom" ]
	then	# they entered their own details
		UPCVSUPSERVER="$UPCVSUPSERVERCUSTOM"
	fi
	
	if [ "$UPCVSUPSERVER" = "fastest_cvsup" -o "$UPCVSUPSERVER" = "automatic" ]
	then	# they want to run fastest_cvsup to workout the cvsup server
		if [ "$FASTESTCVSUPFOUND" != "1" ]
		then	# only do this check once
			if [ "$FASTESTCVSUP" = "" ]
			then	# they do not have fastest_cvsup installed :[
				errCode="4096"
				error
			else
				echo "INFO: Running fastest_cvsup (please wait): `date`"
				FASTESTCVSUPSERVER=`$FASTESTCVSUP -c $UPCVSUPSERVERFASTEST -Q`
				echo "INFO: Using the cvsup server $FASTESTCVSUPSERVER: `date`"
				if [ "$?" = "1" ]
				then	# unable to connect to a server, guessing no network
					if [ "$DEBUG" = "YES" ]
					then	# just report the error and continue
						echo "DEBUG: fastest_cvsup was unable to find a server"
						FASTESTCVSUPFOUND="1"
						UPCVSUPSERVER="debugmode.no.fastest.cvsup"
					else
						errCode="8192"
						error
					fi
				else 
					FASTESTCVSUPFOUND="1"
					UPCVSUPSERVER="$FASTESTCVSUPSERVER"
				fi
			fi # fastest_cvsup installed?
		else 
			# reset the server value
			UPCVSUPSERVER="$FASTESTCVSUPSERVER"
		fi # FASTESTCVSUPFOUND
	fi	# fastest_cvsup

#echo "LASTACTION 1 : $LASTACTION `date`"
	# cvsup the ports
	if [ "$LASTACTION" = "config" ]
	then
		funcCVSPorts
		# move onto the next step
		funcMain
	fi

#echo "LASTACTION 2 : $LASTACTION `date`"
	# cvsup the source
	if [ "$LASTACTION" = "cvsupPorts" ]
	then
		funcCVSSource
		# move onto the next step
		funcMain
	fi

#echo "LASTACTION 3 : $LASTACTION `date`"
	# grab the latest ports index
	if [ "$LASTACTION" = "cvsupSource" ]
	then
		funcFetchIndex
		# move onto the next step
		funcMain
	fi

#echo "LASTACTION 4 : $LASTACTION `date`"
	# update the 'world'
	if [ "$LASTACTION" = "fetchIndex" ]
	then
		funcViewUPDATING
		funcMain
	fi

# NOTE: We could add a mergemaster -p call here (if required)
# NOTE2: A request has for this has been received, so in it goes.
	if [ "$LASTACTION" = "viewUpdate" ]
	then
		funcMergemasterP
		funcMain
	fi

#echo "LASTACTION 4a : $LASTACTION `date`"
	# update the 'world'
	if [ "$LASTACTION" = "mergemasterP" ]
	then
		funcBuildWorld
		funcMain
	fi

#echo "LASTACTION 5 : $LASTACTION `date`"
	if [ "$LASTACTION" = "buildworld" ]
	then
		funcBuildKernel
		funcMain
	fi

#NOTE: we should check here if we need to drop into single user mode or not

#echo "LASTACTION 6 : $LASTACTION `date`"
	if [ "$LASTACTION" = "buildkernel" ]
	then
		funcInstallKernel
		funcMain
	fi

#echo "LASTACTION 7 : $LASTACTION `date`"
	if [ "$LASTACTION" = "installkernel" ]
	then
		funcBuildKernelCustom
		funcMain
	fi

#echo "LASTACTION 8 : $LASTACTION `date`"
	if [ "$LASTACTION" = "buildkernelCustom" ]
	then
		funcInstallKernelCustom
		funcMain
	fi

#echo "LASTACTION 9 : $LASTACTION `date`"
	if [ "$LASTACTION" = "installkernelCustom" ]
	then
		funcInstallWorld
		funcMain
	fi

#echo "LASTACTION 10 : $LASTACTION `date`"
	if [ "$LASTACTION" = "installworld" ]
	then
		funcMergemaster
		funcMain
	fi

#echo "LASTACTION 11 : $LASTACTION `date`"
	if [ "$LASTACTION" = "mergemaster" ]
	then
		funcRebootOption
		funcMain
	fi

#echo "LASTACTION X : $LASTACTION `date`"
	if [ "$LASTACTION" = "rebootRequired" ]
	then # present the option
		funcRebootRequired
	fi

	if [ "$LASTACTION" = "finished" ]
	then
		funcWriteLastUpdate
		funcFinished
	fi

#echo "LASTACTION Y : $LASTACTION `date`"

} # funcMain

# -------
# end of function
# -------

# Process command line switches
while getopts "CDhmqvVj:p:" COMMAND_LINE_ARGUMENT  
do
	case "${COMMAND_LINE_ARGUMENT}" in
		C)
			funcCleanUp ;;
		D)
			DEBUG="YES" ;;
		h) 	
			funcUsage ;;
		v)
			funcVersion ;;
		V)
			funcVersionCheck ;;
		j)
			JVALUE=${OPTARG}
			JOPTION="YES" ;; 
		m)
			MULITUSERBUILD="YES" ;;
		p)
			PROFILECUSTOM="YES"
			PROFILECONF=${OPTARG} ;;
		q)
			NOVERSIONCHECK="YES" 
			QUIETMODE="YES" ;;
		*)
			funcUsage ;;
	esac
done

# store the command line
COMMANDLINE="$@"

# ===========
# Lets start to do some work!
# ===========

# check user
funcUidCheck

# required files and apps check
funcAppsCheck

# check the system mode
funcSystemModeChk
# THIS IS CURRENTLY NOT USED. I have never had an issue updating in multi-user
# mode, however the handbook does meantion updating a system in single user
# mode.
#echo $SYSTEMMODE

# check the value of '-jN'
funcJValueChk 

# present some 'last update' info
if [ -s "$HOME/.sysupdate/lastupdate" -a -s "$PROFILECONF" ]
then	# pull in the info from the update
	funcLastUpdate

	# present the option to continue or not
	if [ "$QUIETMODE" = "YES" ]
	then # we hang for a second or two, then continue
		sleep 3
		continueUpdate="YES"
	else
		# we offer the option to continue or not
		funcContinue
	fi
	if [ "$continueUpdate" != "YES" ]
	then # offer to change settings, run config
		# run the config, they want to make some changes
		funcConfig
	fi
fi

# get to here, there is no lastupdate info, but do we have a config?
if [ -s "$PROFILECONF" ]
then	# load up the config and start the main
	# even if the config has been pulled in before, we pull it in again as it
	# may have been updated from a function above.
	. "$PROFILECONF"

else
	# if we get to here, we assume a clean install or the config files have been removed
	funcConfig
fi

if [ -s "$HOME/.sysupdate/status" ]
then # check for a status file. A status file is use to keep track of the process
	# point for the current update
	. "$HOME/.sysupdate/status"
	unfinishedUpdate="1"
fi

if [ "$unfinishedUpdate" = "1" ]
then	# then pick up from the last successful point
	funcMain
else 
	# begin from the top
	LASTACTION="config"
	funcMain
fi # unfinishedUpdate

if [ "$unfinishedUpdate" != "1" ]
then
	echo "AT THE END AND NOT SURE WHY??"
	echo "Please contact the author to report this issue."

	exit 1
fi


